`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    15:39:30 05/26/2016 
// Design Name: 
// Module Name:    daycounter 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
`define ENABLED 1 // ENABLE indicator
`define DISABLED 0 // EIDABLE indicator
`define INCREMENT 1'b1 // increase by 1
module counter(
   second1,
	second0,
	minute1,
	minute0,
	hour1,
	hour0,
	day1, 
	day0, 
	month1,
	month0,
	year1,
	year0,
	clk, // global clock
	rst_n //active low reset
	);

	// outputs
	output [3:0] second1, second0; //for second
	output [3:0] minute1, minute0; //for minute
	output [3:0] hour1, hour0; //for hour
	output [3:0] day1, day0; //for day
	output [3:0] month1, month0; //for month
	output [3:0] year1, year0; //for year
	// inputs
	input clk; // global clock signal
	input rst_n; // low active reset

	// temporatory nets
	reg load_def_second;
	reg load_def_minute;
	reg load_def_hour;
	reg load_def_day; // enabled to load second value
	reg load_def_month;
	reg load_def_year;

	wire cout_second0, cout_second1; //second carryout
	wire cout_minute0, cout_minute1; //minute carryout
	wire cout_hour0, cout_hour1; //hour carryout
	wire cout_day0, cout_day1; //day carryout
	wire cout_month0, count_month1; //month carryout
	wire cout_year0, cout_year1; //year carryout

	reg [3:0] month_day0, month_day1;

	//**************************************************************
	// second Block
	// return from the end of second 59 to 00
	//**************************************************************
	always @(second1 or second0)
	  if ((second1==4'd5)&&(second0==4'd9))
		 load_def_second = `ENABLED;
	  else
		 load_def_second = `DISABLED;
		 
	upcounter U_second0(
	  .value(second0),  // second 0
	  .carry(cout_second0),  // carry out for second 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(`ENABLED),  // increasing when changing the value of second
	  .load_default(load_def_second),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);
	upcounter U_second1(
	  .value(second1),  // second 1
	  .carry(cout_second1),  // carry out for second 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_second0),  // increasing when second 0 carry out
	  .load_default(load_def_second),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);
	
	//**************************************************************
	// minute Block
	// return from the end of hour 59 to 00
	//**************************************************************
	always @(minute1 or minute0)
	  if ((minute1==4'd5)&&(minute0==4'd9)&&load_def_second)
		 load_def_minute = `ENABLED;
	  else
		 load_def_minute = `DISABLED;
		 
	upcounter U_minute0(
	  .value(minute0),  // minute 0
	  .carry(cout_minute0),  // carry out for minute 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(load_def_second),  // increasing when changing the value of minute
	  .load_default(load_def_minute),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);
	upcounter U_minute1(
	  .value(minute1),  // minute 1
	  .carry(cout_minute1),  // carry out for minute 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_minute0),  // increasing whenminute 0 carry out
	  .load_default(load_def_minute),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);
	
	//**************************************************************
	// hour Block
	// return from the end of hour 23 to 00
	//**************************************************************
	always @(hour1 or hour0)
	  if ((hour1==4'd2)&&(hour0==4'd3)&&load_def_minute)
		 load_def_hour = `ENABLED;
	  else
		 load_def_hour = `DISABLED;
		 
	upcounter U_hour0(
	  .value(hour0),  // hour 0
	  .carry(cout_hour0),  // carry out for hour 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(load_def_minute),  // increasing when changing the value of hour
	  .load_default(load_def_hour),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);
	upcounter U_hour1(
	  .value(hour1),  // hour 1
	  .carry(cout_hour1),  // carry out for hour 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_hour0),  // increasing when hour 0 carry out
	  .load_default(load_def_hour),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);

	
	//**************************************************************
	// Decide the number of days in each month
	// 1,3,5,7,8,10,12: 31 days
	// 4,6,9,11: 30 days
	// 2: 28 days
	//**************************************************************
	always @*
		if({month1,month0}=={4'd0,4'd2})
		begin
			month_day1<=4'd2;
			month_day0<=4'd8;
		end
		else if({month1,month0}=={4'd0,4'd4}||{month1,month0}=={4'd0,4'd6}||
					{month1,month0}=={4'd0,4'd9}|{month1,month0}=={4'd1,4'd1})
		begin
			month_day1<=4'd3;
			month_day0<=4'd0;
		end
		else
		begin
			month_day1<=4'd3;
			month_day0<=4'd1;
		end
	//**************************************************************
	// Day Block
	// return from the end of month(28,30 or31) to 00
	//**************************************************************
	always @(day1 or day0)
	  if ((day1==month_day1)&&(day0==month_day0)&&load_def_hour)
		 load_def_day = `ENABLED;
	  else
		 load_def_day = `DISABLED;

	// counter for day0 and day1
	upcounter U_day0(
	  .value(day0),  // day 0
	  .carry(cout_day0),  // carry out for day 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(load_def_hour),  // always increasing
	  .load_default(load_def_day),  // enable load default value
	  .def_value(4'd1) // default value for counter
	);
	upcounter U_day1(
	  .value(day1),  // day 1
	  .carry(cout_day1),  // carry out for day 1
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_day0),  // increasing when day 0 carry out
	  .load_default(load_def_day),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);

	//**************************************************************
	// Month Block
	// return from the end of year 12 to 01
	//**************************************************************
	always @(month1 or month0)
	  if ((month1==4'd1)&&(month0==4'd2)&&load_def_day)
		 load_def_month = `ENABLED;
	  else
		 load_def_month = `DISABLED;
		 
	upcounter U_Mon0(
	  .value(month0),  // month 0
	  .carry(cout_month0),  // carry out for month 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(load_def_day),  // increasing when changing the value of month
	  .load_default(load_def_month),  // enable load default value
	  .def_value(4'd1) // default value for counter
	);
	upcounter U_Mon1(
	  .value(month1),  // month 1
	  .carry(cout_month1),  // carry out for month 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_month0),  // increasing when month 0 carry out
	  .load_default(load_def_month),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);

	//**************************************************************
	// Year Block
	// return from the end of year 99 to 00
	//**************************************************************

	always @(year1 or year0)
	  if ((year1==4'd9)&&(year0==4'd9)&&load_def_month)
		 load_def_year = `ENABLED;
	  else
		 load_def_year = `DISABLED;

	upcounter U_Year0(
	  .value(year0),  // year 0 of second
	  .carry(cout_year0),  // carry out for year 0
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(load_def_month),  // increasing when changing the value of year
	  .load_default(load_def_year),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);
	upcounter U_Year1(
	  .value(year1),  // year 1 of second
	  .carry(cout_year1),  // carry out for year 1
	  .clk(clk),  // clock
	  .rst_n(rst_n),  // asynchronous low active reset
	  .increase(cout_year0),  // increasing when year 0 carry out
	  .load_default(load_def_year),  // enable load default value
	  .def_value(4'd0) // default value for counter
	);


endmodule
